from generator.yaml import get_yaml, merge_yaml_strings


def test_get_yaml():
    name: str = 'test'
    variables: dict = {
        'mandatories': [{'name': 'mandatories', 'description': 'mandatories'}],
        'optionals': [
            {
                'name': 'optionals',
                'description': 'optionals',
                'default': 'optional_value',
            }
        ],
        'nullables': [{'name': 'nullables', 'description': 'nullables'}],
    }
    resutls = get_yaml(name, variables)
    print(resutls)
    excepted = """test:
  enabled: true
  # mandatories - mandatories
  mandatories: 
  # optionals - optionals
  # optionals: "optional_value"
  # nullables - nullables
  # nullables: 
"""
    assert resutls == excepted


def normalize_lines(text: str) -> str:
    return "\n".join(line.rstrip() for line in text.strip().splitlines())


def test_merge_yaml_strings():
    yaml1 = """gke:
  cluster:
    enabled: true
    # required - required value
    required:
    # optional - optional value
    # optional: "optional"
    # nullable - nullable value
    # nullable:
"""

    yaml2 = """gke:
  helper:
    enabled: true
    # required - required value
    required:
    # optional - optional value
    # optional: "optional"
    # nullable - nullable value
    # nullable:
"""

    expected = """gke:
  cluster:
    enabled: true
    # required - required value
    required:
    # optional - optional value
    # optional: "optional"
    # nullable - nullable value
    # nullable:
  helper:
    enabled: true
    # required - required value
    required:
    # optional - optional value
    # optional: "optional"
    # nullable - nullable value
    # nullable:
"""

    result = merge_yaml_strings(yaml1, yaml2)
    assert normalize_lines(result) == normalize_lines(expected)


def test_merge_yaml_strings_with_blank_lines_inside_blocks():
    yaml1 = """gke:
  cluster:
    enabled: true

    # required - required value
    required:

    # optional - optional value
    # optional: "optional"
"""
    yaml2 = """gke:
  cluster:
    # additional setting
    # another: "value"

  helper:
    enabled: true
"""
    result = merge_yaml_strings(yaml1, yaml2)

    # On vérifie que les lignes vides sont bien conservées dans le bloc cluster
    assert "enabled: true\n\n    # required - required value" in result
    assert "  helper:\n    enabled: true" in result


def test_merge_yaml_strings_with_header_lines():
    yaml1 = """# Generated by terragrunt-generator
# Do not edit manually

gke:
  cluster:
    enabled: true
"""
    yaml2 = """gke:
  helper:
    enabled: true
"""
    result = merge_yaml_strings(yaml1, yaml2)

    # Vérifie que les lignes de header sont bien présentes
    assert "# Generated by terragrunt-generator" in result
    assert "# Do not edit manually" in result
    assert "gke:" in result
    assert "helper:" in result
    assert "cluster:" in result
